package com.reactlibrary

import com.facebook.react.bridge.*
import com.facebook.react.modules.core.RCTNativeAppEventEmitter
import io.inai.android_sdk.*
import io.inai.android_sdk.InaiCrypto
import kotlinx.serialization.decodeFromString
import kotlinx.serialization.json.Json
import org.json.JSONObject

private const val PAYMENT_SUCCESS = "Inai::PAYMENT_SUCCESS"
private const val PAYMENT_ERROR = "Inai::PAYMENT_ERROR"

class InaiCrypto(reactContext: ReactApplicationContext): ReactContextBaseJavaModule(reactContext), InaiCrypto.InaiCryptoEstimateDelegate {

    private var reactContext: ReactApplicationContext

    init {
        this.reactContext = reactContext
    }


    override fun getName(): String {
        return "RNInaiCrypto"
    }

    @ReactMethod
    fun addListener(eventName: String) {
        // Set up any upstream listeners or background tasks as necessary
    }

    @ReactMethod
    fun removeListeners(count: Int) {
        // Remove upstream listeners, stop unnecessary background tasks
    }

    @ReactMethod
    fun getCryptoPurchaseEstimates(
        purchaseDetails: ReadableMap
    ){
        try {
            val inaiCryptoPurchaseDetails = getInaiCryptoPurchaseDetailsFromMap(purchaseDetails)
            currentActivity?.let {
                InaiCrypto.getPurchaseEstimate(inaiCryptoPurchaseDetails,it, this)
            }
        } catch (e: Exception) {
            val wMap = WritableNativeMap()
            wMap.putString("Init Failed", e.message)
            sendEvent(PAYMENT_ERROR, wMap)
        }
    }

    override fun estimateReceived(result: InaiCrypto.InaiCryptoEstimateResult) {
        //  Put only `data` from result into a JSON Object.
        val resultData: JSONObject = result.data

        if (result.status == InaiCrypto.InaiCryptoEstimateStatus.Success) {
            resolvePromise(resultData)
        } else {
            rejectPromise(resultData)
        }
    }

    private fun getInaiCryptoPurchaseDetailsFromMap(map: ReadableMap): InaiCryptoPurchaseDetails {
        val jsonString = JSONObject(map.toHashMap()).toString()
        return Json.decodeFromString(jsonString)
    }

    private fun sendEvent(eventName: String, params: WritableMap) {
        reactContext
            .getJSModule(RCTNativeAppEventEmitter::class.java)
            .emit(eventName, params)
    }

    private fun resolvePromise(resultData: JSONObject) {
        //  Convert JSON Object to WritableMap
        val writableMap = convertJSONToWritableMap(resultData)
        //  Resolve Promise
        sendEvent(PAYMENT_SUCCESS, writableMap)
    }

    private fun rejectPromise(resultData: JSONObject) {
        //  Convert JSON Object to WritableMap
        val writableMap = convertJSONToWritableMap(resultData)
        //  Reject Promise
        sendEvent(PAYMENT_ERROR, writableMap)
    }

    private fun convertJSONToWritableMap(jsonObject: JSONObject): WritableMap {
        val wMap = WritableNativeMap()
        val keys = jsonObject.keys()
        while (keys.hasNext()) {
            val key = keys.next()
            when (val value = jsonObject.get(key)) {
                is String -> {
                    wMap.putString(key, value)
                }
                is Boolean -> {
                    wMap.putBoolean(key, value)
                }
                is JSONObject -> {
                    val map = convertJSONToWritableMap(value)
                    wMap.putMap(key, map)
                }
            }
        }
        return wMap
    }
}