import { NativeModules, NativeEventEmitter } from 'react-native';

const inaiEvents = new NativeEventEmitter(NativeModules.RNInaiSdk);

const removeSubscriptions = () => {
  inaiEvents.removeAllListeners('Inai::PAYMENT_SUCCESS');
  inaiEvents.removeAllListeners('Inai::PAYMENT_ERROR');
};

class InaiCheckout {
  static presentCheckout(config, successCallback,  errorCallback) {
    return new Promise(function(resolve, reject) {
        inaiEvents.addListener('Inai::PAYMENT_SUCCESS', (data) => {
          let resolveFn = successCallback || resolve;
          resolveFn(data);
          removeSubscriptions();
        });
        inaiEvents.addListener('Inai::PAYMENT_ERROR', (data) => {
          let rejectFn = errorCallback || reject;
          rejectFn(data);
          removeSubscriptions();
        });
        NativeModules.RNInaiSdk.presentCheckout(config);
      });
  }

  static addPaymentMethod(config, paymentMethodType, successCallback,  errorCallback) {
    return new Promise(function(resolve, reject) {
        inaiEvents.addListener('Inai::PAYMENT_SUCCESS', (data) => {
          let resolveFn = successCallback || resolve;
          resolveFn(data);
          removeSubscriptions();
        });
        inaiEvents.addListener('Inai::PAYMENT_ERROR', (data) => {
          let rejectFn = errorCallback || reject;
          rejectFn(data);
          removeSubscriptions();
        });
        NativeModules.RNInaiSdk.addPaymentMethod(config, paymentMethodType);
      });
  }

  static payWithPaymentMethod(config, paymentMethodId, successCallback,  errorCallback) {
    return new Promise(function(resolve, reject) {
        inaiEvents.addListener('Inai::PAYMENT_SUCCESS', (data) => {
          let resolveFn = successCallback || resolve;
          resolveFn(data);
          removeSubscriptions();
        });
        inaiEvents.addListener('Inai::PAYMENT_ERROR', (data) => {
          let rejectFn = errorCallback || reject;
          rejectFn(data);
          removeSubscriptions();
        });
        NativeModules.RNInaiSdk.payWithPaymentMethod(config, paymentMethodId);
      });
  }

  static makePayment(config, railCode, paymentDetails, successCallback,  errorCallback) {
    return new Promise(function(resolve, reject) {
        inaiEvents.addListener('Inai::PAYMENT_SUCCESS', (data) => {
          let resolveFn = successCallback || resolve;
          resolveFn(data);
          removeSubscriptions();
        });
        inaiEvents.addListener('Inai::PAYMENT_ERROR', (data) => {
          let rejectFn = errorCallback || reject;
          rejectFn(data);
          removeSubscriptions();
        });
        NativeModules.RNInaiSdk.makePayment(config, railCode, paymentDetails);
      });
  }

  static validateFields(config, paymentMethodOption, paymentDetails, successCallback,  errorCallback) {
    return new Promise(function(resolve, reject) {
        inaiEvents.addListener('Inai::PAYMENT_SUCCESS', (data) => {
          let resolveFn = successCallback || resolve;
          resolveFn(data);
          removeSubscriptions();
        });
        inaiEvents.addListener('Inai::PAYMENT_ERROR', (data) => {
          let rejectFn = errorCallback || reject;
          rejectFn(data);
          removeSubscriptions();
        });
        NativeModules.RNInaiSdk.validateFields(config, paymentMethodOption, paymentDetails);
      });
  }

  static getCardInfo(config, cardNumber, successCallback,  errorCallback) {
    return new Promise(function(resolve, reject) {
        inaiEvents.addListener('Inai::PAYMENT_SUCCESS', (data) => {
          let resolveFn = successCallback || resolve;
          resolveFn(data);
          removeSubscriptions();
        });
        inaiEvents.addListener('Inai::PAYMENT_ERROR', (data) => {
          let rejectFn = errorCallback || reject;
          rejectFn(data);
          removeSubscriptions();
        });
        NativeModules.RNInaiSdk.getCardInfo(config, cardNumber);
      });
  }

  static getApplePayRequestData(paymentMethodOptionsData, successCallback,  errorCallback) {
    return new Promise(function(resolve, reject) {
        inaiEvents.addListener('Inai::PAYMENT_SUCCESS', (data) => {
          let resolveFn = successCallback || resolve;
          resolveFn(data);
          removeSubscriptions();
        });
        inaiEvents.addListener('Inai::PAYMENT_ERROR', (data) => {
          let rejectFn = errorCallback || reject;
          rejectFn(data);
          removeSubscriptions();
        });
        NativeModules.RNInaiSdk.getApplePayRequestData(paymentMethodOptionsData);
      });
  }

  static payWithApplePay(config, paymentRequestData, successCallback,  errorCallback) {
    return new Promise(function(resolve, reject) {
        inaiEvents.addListener('Inai::PAYMENT_SUCCESS', (data) => {
          let resolveFn = successCallback || resolve;
          resolveFn(data);
          removeSubscriptions();
        });
        inaiEvents.addListener('Inai::PAYMENT_ERROR', (data) => {
          let rejectFn = errorCallback || reject;
          rejectFn(data);
          removeSubscriptions();
        });
        NativeModules.RNInaiSdk.payWithApplePay(config, paymentRequestData);
      });
  }

  static initGooglePay(paymentDetails, successCallback, errorCallback) {
    return new Promise(function (resolve, reject) {
      inaiEvents.addListener('Inai::PAYMENT_SUCCESS', (data) => {
        let resolveFn = successCallback || resolve;
        resolveFn(data);
        removeSubscriptions();
      });
      inaiEvents.addListener('Inai::PAYMENT_ERROR', (data) => {
        let rejectFn = errorCallback || reject;
        rejectFn(data);
        removeSubscriptions();
      });
      NativeModules.RNInaiSdk.initGooglePay(paymentDetails);
    });
  }

  static launchGooglePay(config, successCallback, errorCallback) {
    return new Promise(function (resolve, reject) {
      inaiEvents.addListener('Inai::PAYMENT_SUCCESS', (data) => {
        let resolveFn = successCallback || resolve;
        resolveFn(data);
        removeSubscriptions();
      });
      inaiEvents.addListener('Inai::PAYMENT_ERROR', (data) => {
        let rejectFn = errorCallback || reject;
        rejectFn(data);
        removeSubscriptions();
      });
      NativeModules.RNInaiSdk.launchGooglePay(config);
    });
  }
  
  static getInstallmentPlans(config, paymentMethodOption, paymentDetails, successCallback, errorCallback){
    return new Promise(function (resolve, reject){
      inaiEvents.addListener('Inai::PAYMENT_SUCCESS',(data)=>{
        let resolveFn = successCallback || resolve;
        resolveFn(data);
        removeSubscriptions();
      });
      inaiEvents.addListener('Inai::PAYMENT_ERROR', (data) => {
        let rejectFn = errorCallback || reject;
        rejectFn(data);
        removeSubscriptions();
      });
      NativeModules.RNInaiSdk.getInstallmentPlans(config, paymentMethodOption, paymentDetails);
    })
  }
}

export default InaiCheckout;