import Foundation
import inai_ios_sdk
import UIKit
import React
import PassKit

@objc(RNInaiSdk)
class RNInaiSdk: RCTEventEmitter {
    
    let PAYMENT_SUCCESS = "Inai::PAYMENT_SUCCESS"
    let PAYMENT_ERROR = "Inai::PAYMENT_ERROR"
    public static var shared: RNInaiSdk?
    override init() {
        super.init()
        RNInaiSdk.shared = self
    }
    open override func supportedEvents() -> [String] {
        [PAYMENT_SUCCESS, PAYMENT_ERROR]
    }

    private var vc: UIViewController?
    private var applePayCompletion: ((PKPaymentAuthorizationResult) -> Void)?
    private var config: [String: String]?

    @objc(requiresMainQueueSetup)
    override static func requiresMainQueueSetup() -> Bool {
        return false
    }
  
  private func getInaiConfigFromDict(_ config: [String: Any]) -> InaiConfig? {
    do {
      let json = try JSONSerialization.data(withJSONObject: config)
      let decoder = JSONDecoder()
      decoder.keyDecodingStrategy = .useDefaultKeys
      let inaiConfig = try decoder.decode(InaiConfig.self, from: json)
      return inaiConfig
    } catch {
      print(error)
    }
    return nil
  }
  
  
  @objc(presentCheckout:)
  func presentCheckout(config: [String: Any]) -> Void {
    DispatchQueue.main.async {
      if let inaiConfig = self.getInaiConfigFromDict(config),
         let inaiCheckout = InaiCheckout(config: inaiConfig),
         let viewController = RCTPresentedViewController()  {
        self.vc = viewController
        inaiCheckout.presentCheckout(viewController: viewController, delegate: self)
      } else {
        let userInfo = ["message": "Invalid Config"]
          self.sendEvent(withName: self.PAYMENT_ERROR, body:  userInfo)
      }
    }
  }
 
  @objc(addPaymentMethod:paymentMethodType:)
  func addPaymentMethod(config: [String: Any],
                      paymentMethodType: String) -> Void {
    DispatchQueue.main.async {
      if let inaiConfig = self.getInaiConfigFromDict(config),
         let inaiCheckout = InaiCheckout(config: inaiConfig),
         let viewController = RCTPresentedViewController()  {
        self.vc = viewController
        inaiCheckout.addPaymentMethod(type: paymentMethodType,
                                      viewController: viewController,
                                      delegate: self)
      } else {
          let userInfo = ["message": "Invalid Config"]
            self.sendEvent(withName: self.PAYMENT_ERROR, body:  userInfo)
      }
    }
  }

  @objc(payWithPaymentMethod:paymentMethodId:)
  func payWithPaymentMethod(config: [String: Any],
                      paymentMethodId: String) -> Void {
    DispatchQueue.main.async {
      if let inaiConfig = self.getInaiConfigFromDict(config),
         let inaiCheckout = InaiCheckout(config: inaiConfig),
         let viewController = RCTPresentedViewController()  {
        self.vc = viewController
        inaiCheckout.presentPayWithPaymentMethod(paymentMethodId: paymentMethodId,
                                      viewController: viewController,
                                      delegate: self)
      } else {
          let userInfo = ["message": "Invalid Config"]
          self.sendEvent(withName: self.PAYMENT_ERROR, body:  userInfo)
      }
    }
  }


  @objc(makePayment:railCode:paymentDetails:)
  func makePayment(config: [String: Any], railCode: String, paymentDetails: [String: Any]) -> Void {
    DispatchQueue.main.async {
      if let inaiConfig = self.getInaiConfigFromDict(config),
         let inaiCheckout = InaiCheckout(config: inaiConfig),
         let viewController = RCTPresentedViewController()  {
        self.vc = viewController
        inaiCheckout.makePayment(paymentMethodOption: railCode,
                                 paymentDetails: paymentDetails,
                                 viewController: viewController,
                                 delegate: self)
      } else {
          let userInfo = ["message": "Invalid Config"]
            self.sendEvent(withName: self.PAYMENT_ERROR, body:  userInfo)
      }
    }
  }
  
  @objc(validateFields:paymentMethodOption:paymentDetails:)
  func validateFields(config: [String: Any],
                      paymentMethodOption: String, paymentDetails: [String : Any]) -> Void {
    DispatchQueue.main.async {
      if let inaiConfig = self.getInaiConfigFromDict(config),
         let inaiCheckout = InaiCheckout(config: inaiConfig),
         let viewController = RCTPresentedViewController()  {
        self.vc = viewController
        inaiCheckout.validateFields(paymentMethodOption: paymentMethodOption,
                                    paymentDetails: paymentDetails,
                                    viewController: viewController,
                                    delegate: self)
      } else {
          let userInfo = ["message": "Invalid Config"]
            self.sendEvent(withName: self.PAYMENT_ERROR, body:  userInfo)
      }
    }
  }
  
  @objc(getCardInfo:cardNumber:)
  func getCardInfo(config: [String: Any], cardNumber: String) -> Void {
    DispatchQueue.main.async {
      if let inaiConfig = self.getInaiConfigFromDict(config),
         let inaiCheckout = InaiCheckout(config: inaiConfig),
         let viewController = RCTPresentedViewController() {
        self.vc = viewController
        inaiCheckout.getCardInfo(cardNumber: cardNumber,
                                 viewController: viewController,
                                 delegate: self)
      } else {
          let userInfo = ["message": "Invalid Config"]
            self.sendEvent(withName: self.PAYMENT_ERROR, body:  userInfo)
      }
    }
  }
  
  @objc(getApplePayRequestData:)
  func getApplePayRequestData(paymentMethodOptionsData: [String : Any]) -> Void {
    var applePayRequestDict: [String: Any] = [:]

    if let applePayRequestData = InaiCheckout.getApplePayRequestData(paymentMethodOptionsData: paymentMethodOptionsData) {
      applePayRequestDict = [
        "currencyCode" : applePayRequestData.currencyCode,
        "countryCode" : applePayRequestData.countryCode,
        "merchantId" : applePayRequestData.merchantId,
        "productDescription" : applePayRequestData.productDescription,
        "orderAmount" : applePayRequestData.orderAmount,
        "canMakePayments" : applePayRequestData.canMakePayments,
        "canSetupCards" : applePayRequestData.canSetupCards,
        "supportedNetworks" : applePayRequestData.supportedNetworks
      ]
    } else {
      applePayRequestDict["message"] = "no data"
    }
      self.sendEvent(withName: self.PAYMENT_SUCCESS, body:  applePayRequestDict)
  }
  
  @objc func setupApplePay() {
    DispatchQueue.main.async {
      let passLibrary = PKPassLibrary()
      passLibrary.openPaymentSetup()
    }
  }
  
  @objc(payWithApplePay:withApplePaymentRequestData:)
  func payWithApplePay(config: [String:String],
                       withApplePaymentRequestData paymentRequestData: [String: Any]) {
    DispatchQueue.main.async {
      if let viewController = RCTPresentedViewController()  {
        self.config = config
        self.vc = viewController
        let orderSummary = PKPaymentSummaryItem(label: paymentRequestData["productDescription"] as! String,
                                                amount: NSDecimalNumber(string: paymentRequestData["orderAmount"] as? String ?? "0"),
                                                type: .final)
        let paymentSummaryItems = [orderSummary]
        
        // Create a payment request.
        let paymentRequest = PKPaymentRequest()
        paymentRequest.paymentSummaryItems = paymentSummaryItems
        paymentRequest.merchantIdentifier = paymentRequestData["merchantId"] as! String
        paymentRequest.merchantCapabilities = .capability3DS
        
        paymentRequest.countryCode = paymentRequestData["countryCode"] as! String
        paymentRequest.currencyCode = paymentRequestData["currencyCode"] as! String
        
        paymentRequest.supportedNetworks = [.amex, .visa, .masterCard, .discover]
        
        let paymentController = PKPaymentAuthorizationController(paymentRequest: paymentRequest)
        paymentController.delegate = self
        paymentController.present(completion: { (presented: Bool) in
          if presented {
            debugPrint("Presented payment controller")
          } else {
            let userInfo = ["message": "Apple Pay Not Available"]
            self.sendEvent(withName: self.PAYMENT_ERROR, body:  userInfo)
          }
        })
      }
    }
  }
  
  @objc(getInstallmentPlans:paymentMethodOption:paymentDetails:)
  func getInstallmentPlans(config: [String: Any],
                      paymentMethodOption: String, paymentDetails: [String : Any]) {
    DispatchQueue.main.async {
      if let inaiConfig = self.getInaiConfigFromDict(config),
         let inaiCheckout = InaiCheckout(config: inaiConfig),
         let viewController = RCTPresentedViewController()  {
        self.vc = viewController
        inaiCheckout.getInstallmentPlans(paymentMethodOption: paymentMethodOption,
                                    paymentDetails: paymentDetails,
                                    viewController: viewController,
                                    delegate: self)
      } else {
          let userInfo = ["message": "Invalid Config"]
          self.sendEvent(withName: self.PAYMENT_ERROR, body:  userInfo)
      }
    }
  }
}

extension RNInaiSdk: PKPaymentAuthorizationControllerDelegate {
  func paymentAuthorizationController(_ controller: PKPaymentAuthorizationController,
                                      didAuthorizePayment payment: PKPayment,
                                      handler completion: @escaping (PKPaymentAuthorizationResult) -> Void) {
      let config = InaiConfig(token: config!["token"],
                              orderId : config!["orderId"],
                              countryCode: config!["countryCode"] )
      
      if let inaiCheckout = InaiCheckout(config: config) {
          
          let paymentDetails = InaiCheckout.convertPaymentTokenToDict(payment: payment)
          self.applePayCompletion = completion
          
          inaiCheckout.makePayment( paymentMethodOption: "apple_pay",
                                    paymentDetails: paymentDetails,
                                    viewController: self.vc!,
                                    delegate: self)
      }
  }

  func paymentAuthorizationControllerDidFinish(_ controller: PKPaymentAuthorizationController) {
    controller.dismiss {}
  }
}

extension RNInaiSdk: InaiCheckoutDelegate {
  func paymentFinished(with result: Inai_PaymentResult) {
      if let vc = self.vc {
          vc.dismiss(animated: true) {[weak self] in
              guard let self = self else {return}
              if result.status == Inai_PaymentStatus.failed {
                self.sendEvent(withName: self.PAYMENT_ERROR, body:  result.data)
              } else if result.status == Inai_PaymentStatus.canceled {
                self.sendEvent(withName: self.PAYMENT_ERROR, body:  result.data)
              } else {
                  self.sendEvent(withName: self.PAYMENT_SUCCESS, body: result.data)
              }
          }
      }
    }
}

extension RNInaiSdk: InaiValidateFieldsDelegate {
  func fieldsValidationFinished(with result: Inai_ValidateFieldsResult) {
      if let vc = self.vc {
          vc.dismiss(animated: true) {[weak self] in
              guard let self = self else {return}
              if result.status == Inai_ValidateFieldsStatus.failed {
                self.sendEvent(withName: self.PAYMENT_ERROR, body:  result.data)
              } else {
                  self.sendEvent(withName: self.PAYMENT_SUCCESS, body: result.data)
              }
          }
      }
  }
}


extension RNInaiSdk: InaiCardInfoDelegate {
  func cardInfoFetched(with result: Inai_CardInfoResult) {
      if let vc = self.vc {
        vc.dismiss(animated: true) {[weak self] in
            guard let self = self else {return}
            if result.status == Inai_CardInfoStatus.failed {
                self.sendEvent(withName: self.PAYMENT_ERROR, body:  result.data)
            } else {
                self.sendEvent(withName: self.PAYMENT_SUCCESS, body: result.data)
            }
        }
      }
  }
}

extension RNInaiSdk: InaiPaymentMethodDelegate {
  func paymentMethodSaved(with result: Inai_PaymentMethodResult) {
      if let vc = self.vc {
        vc.dismiss(animated: true) {[weak self] in
            guard let self = self else {return}
            if result.status == Inai_PaymentMethodStatus.failed {
                self.sendEvent(withName: self.PAYMENT_ERROR, body:  result.data)
            } else {
                self.sendEvent(withName: self.PAYMENT_SUCCESS, body: result.data)
            }
        }
      }
  }
}

extension RNInaiSdk: InaiInstallmentPlansDelegate {
  func installmentPlansFetched(with result: Inai_InstallmentPlansResult) {
      if let vc = self.vc {
        vc.dismiss(animated: true) {[weak self] in
            guard let self = self else {return}
            if result.status == Inai_InstallmentPlanStatus.failed {
                self.sendEvent(withName: self.PAYMENT_ERROR, body:  result.data)
            } else {
                self.sendEvent(withName: self.PAYMENT_SUCCESS, body: result.data)
            }
        }
      }
  }
}
